<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

require_once 'dao2.php';
require_once 'daoStructure.php';
require_once 'daoSql.php';
require_once 'daoPersistRelations.php';
require_once 'lib/customlib/customMessage/statusResponse.php';

/**
 * Description of daoPersist
 *
 * @author pc3
 */
class daoPersist {

    const MODE_PLAIN = 1;
    const MODE_AUTO = 2;
    const MODE_DEEP = 3;

    private $daoStructure;
    private $statusResponse;
    private $mode = self::MODE_AUTO;


    function __construct() {
        $this->daoStructure = new daoStructure($this->tableName);
    }

    /**
     * @return statusResponse
     */
    function getStatusResponse() {
        return $this->statusResponse;
    }

    function setStatusResponse($statusResponse) {
        $this->statusResponse = $statusResponse;
    }

    /** Setea el modo de instanciacion en plano */
    function setPlainMode(){ $this->mode = self::MODE_PLAIN; }
    /** Setea el modo de instanciacion en auto */
    function setAutoMode(){ $this->mode = self::MODE_AUTO; }
    /** Setea el modo de instanciacion en deep */
    function setDeepMode(){ $this->mode = self::MODE_DEEP; }


    /**
     * Devuelve objeto extendido instanciado con el modo en que este seteado,
     * por defecto, el modo es Auto, + statusResponse con resultado de la operacion.
     *
     * @param int $id
     * @return daoPersist
     */
    protected function get($id) {
        
        $_ret = false;
        try {
            $daoSql = new daoSql();
            $dbConn = $daoSql->dbConnect();

            if (!DbConfig::isError($dbConn)) {

                if ($this->daoStructure) {

                    $query = $this->daoStructure->getQueryForInstantiate($id);

                    $daoResult = $daoSql->select($query);

                    if ($daoResult->getStatusResponse()->isOk()) {

                        $resultSet = $daoResult->getNextRow();

                        $_ret = $this->instantiateFromResultset($resultSet);
                                                
                    } else {
                        $statusResponse = $daoResult->getStatusResponse();
                    }

                } else {
                    
                    $statusResponse = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NODATASTRUCURE4ENTITY, $this->tableName);
                }
            } else {

                $statusResponse = statusResponse::getInstanceForMDBError($dbConn);
            }

        } catch (Exception $e) {
            
            FB::info($e); // no quitar. buena para debug.
            $statusResponse = statusResponse::getInstanceFromException($e);

        } finally {

            if (! $_ret) {
                $_ret = clone $this;
                $_ret->setStatusResponse($statusResponse);
            }
        }

        return $_ret;
    }

    /**
     * Devuelve objeto extendido instanciado con el modo en que este seteado,
     * por defecto, el modo es Auto, + statusResponse con resultado de la operacion.
     *
     * @param string $query
     * @return daoPersist
     */
    protected function getUniqueResultFromQuery($query) {

        $_ret = false;
        try {
            $daoSql = new daoSql();
            $dbConn = $daoSql->dbConnect();

            if (!DbConfig::isError($dbConn)) {

                if ($this->daoStructure) {

                    $daoResult = $daoSql->select($query);

                    if ($daoResult->getStatusResponse()->isOk()) {
                        
                        if($daoResult->getCursor()->numRows() === 1) {
                            $resultSet = $daoResult->getNextRow();

                            $_ret = $this->instantiateFromResultset($resultSet);

                        }else{
                            if($daoResult->getCursor()->numRows() > 1)
                                $statusResponse = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NOTUNIQUERESULT);
                            else
                                $statusResponse = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NORESULT);
                        }

                    } else {
                        $statusResponse = $daoResult->getStatusResponse();
                    }

                } else {

                    $statusResponse = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NODATASTRUCURE4ENTITY, $this->tableName);
                }
            } else {

                $statusResponse = statusResponse::getInstanceForMDBError($dbConn);
            }

        } catch (Exception $e) {

            FB::info($e); // no quitar. buena para debug.
            $statusResponse = statusResponse::getInstanceFromException($e);

        } finally {

            if (! $_ret) {
                $_ret = clone $this;
                $_ret->setStatusResponse($statusResponse);
            }
        }

        return $_ret;
    }


    /**
     * Devuelve objeto instanciado en modo plain, sin ninguna relación.
     *  + statusResponse con resultado de la operacion.
     *
     * @param int $id
     * @return daoPersist
     */
    protected function getPlain($id){
        $this->setPlainMode();
        return $this->get($id);
    }

    /**
     * Devuelve objeto instanciado en modo auto, con las relaciones en auto instanciadas.
     *  + statusResponse con resultado de la operacion.
     *
     * @param int $id
     * @return daoPersist
     */
    protected function getAuto($id){
        $this->setAutoMode();
        return $this->get($id);
    }

    /**
     * Devuelve objeto extendido instanciado con todas las relaciones, y dependiendo de la configuración
     * la instancia de manera deep o simple, + statusResponse con resultado de la operacion.
     *
     * @param int $id
     * @return daoPersist
     */
    protected function getDeep($id) {
        $this->setDeepMode();
        return $this->get($id);
    }

    /**
     * Instancia la entidad con sus relaciones desde un resultset
     *
     * @param $resultSet
     * @return bool|daoPersist
     */
    public function instantiateFromResultset($resultSet) {
        $_ret = false;
        
        if ($resultSet) {
            $_ret = $this->getRsInstantiatedObj($resultSet);
            
            if($this->isAutoMode())
                $this->setAutoRelations($_ret, $resultSet);
            elseif($this->isDeepMode()) {
                $this->setDeepRelations($_ret);
            }
            
            $statusResponse = statusResponse::getInstanceOk();
        } else {

            $statusResponse = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NOTFOUND);
        }

        if (!$_ret)
            $_ret = clone $this;
        
        $_ret->setStatusResponse($statusResponse);
        
        return $_ret;
    }

    /**
     *
     * @return statusResponse
     */
    public function Insert() {
        $_primaryKey = $this->daoStructure->getPrimaryKeyField();
        $this->$_primaryKey = null;

        $field_values = $this->daoStructure->getFieldTypesArray();
        foreach ($field_values as $prop => $type) {
            $field_values[$prop] = $this->$prop;
        }

        try {
            $daoSql = new daoSql();
            $daoSql->dbConnect();

            $query = $this->daoStructure->getQueryForInsert($field_values);

            $daoResponse = $daoSql->exec($query);
            $_ret = $daoResponse->getStatusResponse();

            if ($_ret->isOk()) {
                $_ret = statusResponse::getInstanceOk();

                $this->$_primaryKey = $daoSql->getLastInsertID($this->daoStructure);
            } else {
                $_detail = $_ret->getStatusMsg();
                $_debugInfo = $_ret->getDebugInfo();

                $_ret = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NOTINSERT);
                $_ret->setStatusDetail($_detail);
                $_ret->setDebugInfo($_debugInfo);
            }
        } catch (Exception $e) {
            $_ret = statusResponse::getInstanceFromException($e);
        }

        $this->setStatusResponse($_ret);

        return $this->getStatusResponse();
    }

    /**
     *
     * @return statusResponse
     */
    public function Update() {
        $_primaryKey = $this->daoStructure->getPrimaryKeyField();

        $where_field = array($_primaryKey => $this->$_primaryKey);

        $field_values = $this->daoStructure->getFieldTypesArray();
        foreach ($field_values as $prop => $type) {
            $field_values[$prop] = $this->$prop;
        }

        try {
            $daoSql = new daoSql();
            $daoSql->dbConnect();

            $query = $this->daoStructure->getQueryForUpdate($where_field, $field_values);
            $daoResponse = $daoSql->exec($query);

            $_ret = $daoResponse->getStatusResponse();

            if ($_ret->isOk()) {
                $_ret = statusResponse::getInstanceOk();
            } else {
                $_detail = $_ret->getStatusMsg();
                $_debugInfo = $_ret->getDebugInfo();

                $_ret = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NOTUPDATE);
                $_ret->setStatusDetail($_detail);
                $_ret->setDebugInfo($_debugInfo);
            }
        } catch (Exception $e) {
            $_ret = statusResponse::getInstanceFromException($e);
        }

        $this->setStatusResponse($_ret);

        return $this->getStatusResponse();
    }

    /**
     * Persiste la entidad hacia la base de datos.
     * Si tiene primary key lo guarda y sino lo inserta.
     *
     * @return statusResponse
     */
    public function Persist() {
        $_primaryKey = $this->daoStructure->getPrimaryKeyField();

        if ($this->$_primaryKey) {
            $_ret = $this->Update();
        } else {
            $_ret = $this->Insert();
        }

        return $_ret;
    }

    /**
     * @param $_id
     * @return statusResponse
     */
    public function Delete($_id) {

        try {
            $daoSql = new daoSql();
            $daoSql->dbConnect();

            $query = $this->daoStructure->getQueryForDelete($_id);
            $daoResponse = $daoSql->exec($query);

            $_ret = $daoResponse->getStatusResponse();

            if ($_ret->isOk()) {
                $_ret = statusResponse::getInstanceOk();
            } else {
                $_detail = $_ret->getStatusMsg();
                // TODO, revisar como propagar msg de error para seguimiento.

                $_ret = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NOTDELETE);
                $_ret->setStatusDetail($_detail);
            }
        } catch (Exception $e) {
            
            $_ret = statusResponse::getInstanceFromException($e);
        }

        $this->setStatusResponse($_ret);

        return $this->getStatusResponse();
    }

    /**
     * Setea la relacion de la entidad con el nombre de la propiedad
     * @param $propertyName
     * @return statusResponse
     */
    public function setRelation($propertyName, $resultset=null) {
        $_ret = statusResponse::getInstanceOk();

        $oto_relations = $this->daoStructure->getOneToOneRelations();
        $otm_relations = $this->daoStructure->getOneToManyRelations();

        $relations = new daoPersistRelations($oto_relations, $otm_relations);

        try {
            
            $daoPersistRel = $relations->getRelation($propertyName);
            if ($daoPersistRel) {
                $daoPersistRel->setObject($this, $resultset);
                $daoPersistRel->setPropertyInParentObject($this);
            } else {
                $_ret = statusResponse::getInstanceFromTableMsgCode(msgTable::ERROR_DATABASE_DAOPERSIST_NOTFOUND);
            }

        } catch (Exception $e) {
            $_ret = statusResponse::getInstanceFromException($e);
        }

        return $_ret;
    }


    /**
     * Ejecuta el query pasado como parametro
     *
     * @param $query
     * @return daoResponse
     */
    protected function getList($where=null, $order=null){
        try {
            $daoSql = new daoSql();
            $daoSql->dbConnect();

            $query = $this->daoStructure->getQueryForList($where,$order);

            $_ret = $daoSql->select($query);

        } catch (Exception $e) {
            $_status = statusResponse::getInstanceFromException($e);

            $_ret = new daoResponse(null);
            $_ret->setStatusResponse($_status);
        }

        return $_ret;
    }

    public function getFieldType($fieldName) {
        return $this->daoStructure->getFieldType($fieldName);
    }

    public function getPrimaryKeyValue(){
        $_primaryKey = $this->daoStructure->getPrimaryKeyField();
        return $this->$_primaryKey;
    }


    /*******************  Metodos protegidos/privados  **********************/


    protected function isPlainMode(){ return ($this->mode===self::MODE_PLAIN); }
    protected function isAutoMode(){ return ($this->mode===self::MODE_AUTO); }
    protected function isDeepMode(){ return ($this->mode===self::MODE_DEEP); }

    protected function getRsInstantiatedObj(&$resultSet) {

        // Clonamos el objeto hijo para retornar la clase que corresponda.
        $_ret = clone $this;
        
        $_fields = $this->daoStructure->getFieldTypesArray();        

        foreach ($_fields as $prop => $type) {
            $value = $resultSet[$prop];
            $this->daoStructure->setValueByType($_ret, $prop, $value);
        }

        return $_ret;
    }

    private function setAutoRelations($parentObject, $resultset) {
        $oto_relations = $this->daoStructure->getOneToOneRelations();
        $otm_relations = $this->daoStructure->getOneToManyRelations();
        
        $relations = new daoPersistRelations($oto_relations, $otm_relations);
        $relations->setAutoInstantiateRelations($parentObject, $resultset);
        
        $relationList = array();
        $oneToOneList = $relations->getOneToOne_autoInstantiateRelationList();
        $oneToManyList = $relations->getOneToMany_autoInstantiateRelationList();

        if (is_array($oneToOneList))
            $relationList = array_merge($relationList, $oneToOneList);
        if (is_array($oneToManyList))
            $relationList = array_merge($relationList, $oneToManyList);

        foreach ($relationList as $daoPersistRelationItem) {
            // hay una relacion
            if (is_object($daoPersistRelationItem)) {
                $daoPersistRelationItem->setPropertyInParentObject($parentObject);
            }
        }
    }

    private function setDeepRelations($parentObject) {
        $oto_relations = $this->daoStructure->getOneToOneRelations();
        $otm_relations = $this->daoStructure->getOneToManyRelations();

        $relations = new daoPersistRelations($oto_relations, $otm_relations);

        $relationList = array();
        if (is_array($oto_relations))
            $relationList = array_merge($relationList, $oto_relations);
        if (is_array($otm_relations))
            $relationList = array_merge($relationList, $otm_relations);
        foreach ($relationList as $relationObj) {
            // hay una relacion
            $daoPersistRelationItem = $relations->getRelation($relationObj->propertyName);
            if (is_object($daoPersistRelationItem)) {
                $daoPersistRelationItem->setObjectForDeepInstance($parentObject);
                $daoPersistRelationItem->setPropertyInParentObject($parentObject);
            }
        }
    }

}
